# -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
# ***** BEGIN LICENSE BLOCK *****
# Version: MPL 1.1/GPL 2.0/LGPL 2.1
#
# The contents of this file are subject to the Mozilla Public License Version
# 1.1 (the "License"); you may not use this file except in compliance with
# the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS IS" basis,
# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
# for the specific language governing rights and limitations under the
# License.
#
# The Original Code is Mozilla Communicator client code, released
# March 31, 1998.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998-1999
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Markus Hossner <markushossner@gmx.de>
#   Postbox, Inc. <contact@postbox-inc.com>
#
# Alternatively, the contents of this file may be used under the terms of
# either the GNU General Public License Version 2 or later (the "GPL"), or
# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
# in which case the provisions of the GPL or the LGPL are applicable instead
# of those above. If you wish to allow use of your version of this file only
# under the terms of either the GPL or the LGPL, and not to allow others to
# use your version of this file under the terms of the MPL, indicate your
# decision by deleting the provisions above and replace them with the notice
# and other provisions required by the GPL or the LGPL. If you do not delete
# the provisions above, a recipient may use your version of this file under
# the terms of any one of the MPL, the GPL or the LGPL.
#
# ***** END LICENSE BLOCK *****

/* This is where functions related to displaying the headers for a selected message in the
   message pane live. */

////////////////////////////////////////////////////////////////////////////////////
// Warning: if you go to modify any of these JS routines please get a code review from
// scott@scott-macgregor.org. It's critical that the code in here for displaying
// the message headers for a selected message remain as fast as possible. In particular,
// right now, we only introduce one reflow per message. i.e. if you click on a message in the thread
// pane, we batch up all the changes for displaying the header pane (to, cc, attachements button, etc.)
// and we make a single pass to display them. It's critical that we maintain this one reflow per message
// view in the message header pane.
////////////////////////////////////////////////////////////////////////////////////

const msgHeaderParserContractID       = "@mozilla.org/messenger/headerparser;1";
const abAddressCollectorContractID   = "@mozilla.org/addressbook/services/addressCollecter;1";
const kPersonalAddressbookUri        = "moz-abmdbdirectory://abook.mab";
const kRDFServiceContractID          = "@mozilla.org/rdf/rdf-service;1";

const kLargeIcon = 32;
const kSmallIcon = 16;

const kInlineQuotes = 2;
const kNonInlineQuotes = 1;
const kNoQuotes = 0;

var gShowOrganization = false;
var gShowUserAgent = false;
var gShowReferences = false;
var gShowMessageId = false;
var gExtraExpandedHeaders;
var gMinNumberOfHeaders = 0;
var gDummyHeaderIdIndex = 0;
var gBuildAttachmentsForCurrentMsg = false;
var gBuildAttachmentPopupForCurrentMsg = true;
var gBuiltHeaderView = false;
var gOpenLabel;
var gOpenLabelAccesskey;
var gSaveLabel;
var gSaveLabelAccesskey;
var gDetachLabel;
var gDeleteLabel;
var gDetachLabelAccesskey;
var gDeleteLabelAccesskey;
var gMessengerBundle;
var gProfileDirURL;
var gPersonalAddressBookDirectory; // used for determining if we want to show just the display name in email address nodes

var msgHeaderParser = Components.classes[msgHeaderParserContractID].getService(Components.interfaces.nsIMsgHeaderParser);
var abAddressCollector = null;

// other components may listen to on start header & on end header notifications for each message we display
// to do that you need to add yourself to our gMessageListeners array with object that has two properties:
// onStartHeaders and onEndHeaders.
var gMessageListeners = new Array;

// For every possible "view" in the message pane, you need to define the header names you want to
// see in that view. In addition, include information describing how you want that header field to be
// presented. i.e. if it's an email address field, if you want a toggle inserted on the node in case
// of multiple email addresses, etc. We'll then use this static table to dynamically generate header view entries
// which manipulate the UI.
// When you add a header to one of these view lists you can specify the following properties:
// name: the name of the header. i.e. "to", "subject". This must be in lower case and the name of the
//       header is used to help dynamically generate ids for objects in the document. (REQUIRED)
// useToggle:      true if the values for this header are multiple email addresses and you want a
//                 a toggle icon to show a short vs. long list (DEFAULT: false)
// useShortView:   (only works on some fields like From). If the field has a long presentation and a
//                 short presentation we'll use the short one. i.e. if you are showing the From field and you
//                 set this to true, we can show just "John Doe" instead of "John Doe <jdoe@netscape.net>".
//                 (DEFAULT: false)
//
// outputFunction: this is a method which takes a headerEntry (see the definition below) and a header value
//                 This allows you to provide your own methods for actually determining how the header value
//                 is displayed. (DEFAULT: updateHeaderValue which just sets the header value on the text node)

var gHeaderList = [ {name:"subject", outputFunction:OutputSubject},
                            {name:"from", outputFunction:OutputEmailAddresses},
                            {name:"sender", outputFunction:OutputEmailAddresses},
                            {name:"reply-to", useToggle:true, outputFunction:OutputEmailAddresses},
                            {name:"date"},
                            {name:"to", useToggle:true, outputFunction:OutputEmailAddresses},
                            {name:"participants", useToggle:true, outputFunction:OutputEmailAddresses},
                            {name:"cc", useToggle:true, outputFunction:OutputEmailAddresses},
                            {name:"bcc", useToggle:true, outputFunction:OutputEmailAddresses},
                            {name:"newsgroups", outputFunction:OutputNewsgroups},
                            {name:"content-base"},
                            {name:"tags", outputFunction:OutputTags} ];

// Postbox does not use gViewAllHeaders, gCollapsedHeaderList, gExpandedHeaderList, gExpandedHeaderView or gCollapsedHeaderView anymore. Leaving the variable declarations alone for 
// add on compatibility.
var gViewAllHeaders = false;
var gExpandedHeaderList = [];
var gCollapsedHeaderList = [];
var gExpandedHeaderView  = {};
var gCollapsedHeaderView = {};

// Now, for each view the message pane can generate, we need a global table of headerEntries. These
// header entry objects are generated dynamically based on the static date in the header lists (see above)
// and elements we find in the DOM based on properties in the header lists.
var gHeaderView  = {};

// currentHeaderData --> this is an array of header name and value pairs for the currently displayed message.
//                       it's purely a data object and has no view information. View information is contained in the view objects.
//                       for a given entry in this array you can ask for:
// .headerName ---> name of the header (i.e. 'to'). Always stored in lower case
// .headerValue --> value of the header "johndoe@netscape.net"
var currentHeaderData = {};

// For the currently displayed message, we store all the attachment data. When displaying a particular
// view, it's up to the view layer to extract this attachment data and turn it into something useful.
// For a given entry in the attachments list, you can ask for the following properties:
// .contentType --> the content type of the attachment
// url --> an imap, or mailbox url which can be used to fetch the attachment
// uri --> an RDF URI which refers to the message containig the attachment
// isExternalAttachment --> boolean flag stating whether the attachment is an attachment which is a URL that refers to the attachment location
var currentAttachments = new Array();

// createHeaderEntry --> our constructor method which creates a header Entry
// based on an entry in one of the header lists. A header entry is different from a header list.
// a header list just describes how you want a particular header to be presented. The header entry
// actually has knowledge about the DOM and the actual DOM elements associated with the header.
// headerListInfo --> entry from a header list.
function createHeaderEntry(headerListInfo)
{
  var useShortView = false;
  this.enclosingBox = document.getElementById(headerListInfo.name + 'Box');
  this.textNode = document.getElementById(headerListInfo.name + 'Value');
  this.isValid = false;

  if ("useShortView" in headerListInfo)
  {
    useShortView = headerListInfo.useShortView;
    if (useShortView)
      this.enclosingBox = this.textNode;
    else
      this.enclosingBox.emailAddressNode = this.textNode;
  }

  if ("useToggle" in headerListInfo)
  {
    this.useToggle = headerListInfo.useToggle;
    if (this.useToggle) // find the toggle icon in the document
    {
      this.toggleIcon = this.enclosingBox.childNodes[0].toggleIcon;
      this.longTextNode = this.enclosingBox.childNodes[1].longEmailAddresses;
      this.textNode = this.enclosingBox.childNodes[1].emailAddresses;
    }
  }
  else
    this.useToggle = false;

  if (this.textNode)
    this.textNode.useShortView = useShortView;

  if ("outputFunction" in headerListInfo)
    this.outputFunction = headerListInfo.outputFunction;
  else
    this.outputFunction = updateHeaderValue;
}

function initializeHeaderViewTables()
{
  // iterate over each header in our header list arrays and create header entries
  // for each one. These header entries are then stored in the appropriate header table
  var index;
  for (index = 0; index < gHeaderList.length; index++)
  {
    var headerName = gHeaderList[index].name;
    gHeaderView[headerName] = new createHeaderEntry(gHeaderList[index]);
  }

  var extraHeaders = gExtraExpandedHeaders.split(' ');
  for (index = 0; index < extraHeaders.length; index++)
  {
    var extraHeader = extraHeaders[index];
    if (extraHeader)
    {
      gHeaderView[extraHeader.toLowerCase()] = new createNewHeaderView(extraHeader, extraHeader + ':');
      gExpandedHeaderView[extraHeader.toLowerCase()] = gHeaderView[extraHeader.toLowerCase()]; // for add-ons compatibility only
    }
  }

  if (gShowOrganization)
  {
    var organizationEntry = {name:"organization", outputFunction:updateHeaderValue};
    gHeaderView[organizationEntry.name] = new createHeaderEntry(organizationEntry);
  }

  if (gShowUserAgent)
  {
    var userAgentEntry = {name:"user-agent", outputFunction:updateHeaderValue};
    gHeaderView[userAgentEntry.name] = new createHeaderEntry(userAgentEntry);
  }

  if (gShowMessageId)
  {
    var messageIdEntry = {name:"message-id", outputFunction:OutputMessageIds};
    gHeaderView[messageIdEntry.name] = new createHeaderEntry(messageIdEntry);
  }
}

function OnLoadMsgHeaderPane()
{
  // HACK...force our XBL bindings file to be load before we try to create our first xbl widget....
  // otherwise we have problems.
  document.loadBindingDocument('chrome://messenger/content/mailWidgets.xml');
  document.loadBindingDocument('chrome://messenger/content/pbSearchBox.xml');

  // load any preferences that at are global with regards to
  // displaying a message...
  gShowUserAgent = pref.getBoolPref("mailnews.headers.showUserAgent");
  gMinNumberOfHeaders = pref.getIntPref("mailnews.headers.minNumHeaders");
  gShowOrganization = pref.getBoolPref("mailnews.headers.showOrganization");
  gShowReferences = pref.getBoolPref("mailnews.headers.showReferences");
  gShowMessageId = pref.getBoolPref("mailnews.headers.showMessageId");
  gExtraExpandedHeaders = pref.getCharPref("mailnews.headers.extraExpandedHeaders");

  initializeHeaderViewTables();

  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  observerService.addObserver(gTagChangeListener, "pb-tag-changed", false);
  
  // dispatch an event letting any listeners know that we have loaded the message pane
  var event = document.createEvent('Events');
  event.initEvent('messagepane-loaded', false, true);
  var headerViewElement = document.getElementById("msgHeaderView");
  headerViewElement.dispatchEvent(event);
}

function OnUnloadMsgHeaderPane()
{
  var observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
  observerService.removeObserver(gTagChangeListener, "pb-tag-changed");
  
  // dispatch an event letting any listeners know that we have unloaded the message pane
  var event = document.createEvent('Events');
  event.initEvent('messagepane-unloaded', false, true);
  var headerViewElement = document.getElementById("msgHeaderView");
  headerViewElement.dispatchEvent(event);
}

/**
 * Called by nsIMsgDBViewCommandUpdate::PrepareToLoadMessage
 * when we are about to display a message or conversation. 
 * Clear out & initialize any display state.
 */
function PrepareMsgPaneForDisplay()
{
  // used to style the message pane box differently when displaying a message vs. the start page
  document.getElementById('messagepanebox').setAttribute('message', true);
  ClearCurrentHeaders();
  ClearAttachmentList();
  clearAttachmentHeader();
  gBuildAttachmentsForCurrentMsg = false;
  gBuildAttachmentPopupForCurrentMsg = true;
  messageHeaderSink._conversationMode = false;
  gMessageNotificationBar.clearMsgNotifications();
  try {
    // onBeginMessageLoad can throw after the 3-pane layout has changed
    gDataPane.onBeginMessageLoad(true);
  }
  catch (ex) {}
}

// The messageHeaderSink is the class that gets notified of a message's headers as we display the message
// through our mime converter.
var messageHeaderSink = {
    _conversationMode: false,
    onStartHeaders: function(aConversation)
    {
      gBuiltHeaderView = false;
      for (index in gMessageListeners)
        gMessageListeners[index].onStartHeaders();
      document.getElementById('subjectOverflowLabel').hidden = true;
    },

    onEndHeaders: function()
    {
      CheckNotify();
      ClearHeaderView(gHeaderView);
      ClearHeaderView(gExpandedHeaderView);

      ShowMessageHeaderPane();
      UpdateMessageHeaders();

      if (!this._conversationMode)
      {
        // push the from contact info into the data pane
        var fromNode = document.getElementById('fromAddressNode');

        // there are no special headers for linkedin emails
        gDataPane.updateContact(fromNode._displayName, fromNode._emailAddress, fromNode._card, 
                                currentHeaderData['x-facebook-notify'] ? getFacebookIdFromHeader(currentHeaderData['x-facebook-notify'].headerValue) : '',
                                currentHeaderData['x-twittersenderid'] ? currentHeaderData['x-twittersenderid'].headerValue : '');
      }
      for (index in gMessageListeners)
        gMessageListeners[index].onEndHeaders();
    },

    processHeaders: function(headerNameEnumerator, headerValueEnumerator, aConversation)
    {
      if (this._conversationMode)
        return;

      this.onStartHeaders();
      const kMailboxSeparator = ", ";
      var index = 0;

      var extraHeaders = gExtraExpandedHeaders.split(' ');

      // conversation mode ignores the message headers
      while (headerNameEnumerator.hasMore())
      {
        var header = new Object;
        header.headerValue = headerValueEnumerator.getNext();
        header.headerName = headerNameEnumerator.getNext();

        // for consistancy sake, let's force all header names to be lower case so
        // we don't have to worry about looking for: Cc and CC, etc.
        var lowerCaseHeaderName = header.headerName.toLowerCase();
        
        // if we are in conversation mode, be sure to include headers if they are in gExtraExpandedHeaders for add on compatibility reasons
        var extraHeaderForConversation = false;
        for (index = 0; aConversation && !extraHeaderForConversation && index < extraHeaders.length; index++)
        {
          if (lowerCaseHeaderName == extraHeaders[index].toLowerCase())
            extraHeaderForConversation = true;
        }
        if (aConversation && !extraHeaderForConversation)
          continue;

        // if we have an x-mailer or x-mimeole string, put it in the user-agent slot which we know how to handle
        // already.
        if (lowerCaseHeaderName == "x-mailer" || lowerCaseHeaderName == "x-mimeole")
          lowerCaseHeaderName = "user-agent";

        if (this.mDummyMsgHeader)
        {
          if (lowerCaseHeaderName == "from")
            this.mDummyMsgHeader.author = header.headerValue;
          else if (lowerCaseHeaderName == "to")
            this.mDummyMsgHeader.recipients = header.headerValue;
          else if (lowerCaseHeaderName == "cc")
            this.mDummyMsgHeader.ccList = header.headerValue;
          else if (lowerCaseHeaderName == "subject")
            this.mDummyMsgHeader.subject = header.headerValue;
          else if (lowerCaseHeaderName == "reply-to")
            this.mDummyMsgHeader.replyTo = header.headerValue;
          else if (lowerCaseHeaderName == "message-id")
            this.mDummyMsgHeader.messageId = header.headerValue;
        }
        // according to RFC 2822, certain headers
        // can occur "unlimited" times
        if (lowerCaseHeaderName in currentHeaderData)
        {
          // sometimes, you can have multiple To or Cc lines....
          // in this case, we want to append these headers into one.
          if (lowerCaseHeaderName == 'to' || lowerCaseHeaderName == 'cc')
            currentHeaderData[lowerCaseHeaderName].headerValue = currentHeaderData[lowerCaseHeaderName].headerValue + ',' + header.headerValue;
          else {
            // use the index to create a unique header name like:
            // received5, received6, etc
            currentHeaderData[lowerCaseHeaderName + index++] = header;
          }
        }
        else
          currentHeaderData[lowerCaseHeaderName] = header;
      } // while we have more headers to parse

      // process message tags & the star state as if they were headers in the message
      SetTagHeader(aConversation);
      if (aConversation)
        SetConversationHeaders();
      else
      {
        // if this isn't a conversation and we did't get a subject, give it a default subject of "(no subject)"
        if (!currentHeaderData["subject"])
          currentHeaderData["subject"] = {headerName: "subject", headerValue: gMessengerBundle.getString("noSubject")};
      }

      SetReconstructLink();
      
      if (("from" in currentHeaderData) && ("sender" in currentHeaderData) && msgHeaderParser)
      {
        var senderMailbox = kMailboxSeparator + msgHeaderParser.extractHeaderAddressMailboxes(null,
                            currentHeaderData.sender.headerValue) + kMailboxSeparator;
        var fromMailboxes = kMailboxSeparator + msgHeaderParser.extractHeaderAddressMailboxes(null,
                            currentHeaderData.from.headerValue) + kMailboxSeparator;
        if (fromMailboxes.indexOf(senderMailbox) >= 0)
          delete currentHeaderData.sender;
      }
      this._conversationMode = aConversation; // avoid processing headers for other msgs in the conversation
      this.onEndHeaders();
    },

    handleAttachment: function(contentType, url, displayName, uri, isExternalAttachment)
    {
      // presentation level change....don't show vcards as external attachments in the UI.
      // libmime already renders them inline.
      var msgHdr = null;
      try
      {
        msgHdr = messenger.messageServiceFromURI(uri).messageURIToMsgHdr(uri);
      }
      catch (ex) {}

      if (contentType == "text/x-vcard")
      {
        var inlineAttachments = pref.getBoolPref("mail.inline_attachments");
        var displayHtmlAs = pref.getIntPref("mailnews.display.html_as");
        if (inlineAttachments && !displayHtmlAs)
        {
          return;
        }
      }

      currentAttachments.push (new createNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment));
      // if we have an attachment, set the MSG_FLAG_ATTACHMENT flag on the hdr
      // this will cause the "message with attachment" icon to show up
      // in the thread pane
      // we only need to do this on the first attachment
      var numAttachments = currentAttachments.length;
      if (numAttachments == 1) {
        // we also have to enable the File/Attachments menuitem
        var node = document.getElementById("fileAttachmentMenu");
        if (node)
          node.removeAttribute("disabled");

        try {
          // convert the uri into a hdr
          if (msgHdr)
            msgHdr.markHasAttachments(true);
        }
        catch (ex) {}
      }
    },

    onEndAllAttachments: function() {},

    onEndMsgDownload: function(aUrl, aDocument)
    {
      // if we ever need to know if all of the frames inside the message pane have been loaded, 
      // it's when onEndMsgDownload is called with message pane's contentDocument.
      // NOTE:  this is also true the message is not part of a conversation
      // see bug #4831 for details
      var msgPane = document.getElementById('messagepane');
      if (msgPane.contentDocument == aDocument)
      {
        var showConversationPhotos = pref.getBoolPref('pb.display_contact_photos');
        var fromValues = aDocument.getElementsByClassName('from-value');
        var conversationPhotoCells;
        // only look for the conversation-contact-photo-container elements if this is a conversation
        // and use fromValues.length (not this._conversation) to determine that.
        // see bug #4651 for details
        if (fromValues.length)
        {
          conversationPhotoCells = aDocument.getElementsByClassName('conversation-contact-photo-insertnode');
          msgPane.contentWindow.addEventListener('overflow', 
                                                 function (event) {if (event.detail == 0) msgPane.contentWindow.document.body.setAttribute('overflow', 'true'); }, 
                                                 false);
          msgPane.contentWindow.addEventListener('underflow', 
                                                 function (event) {if (event.detail == 0) msgPane.contentWindow.document.body.removeAttribute('overflow') }, 
                                                 false);
        }
        for (var index = 0; showConversationPhotos && index < fromValues.length; index++)
        {
          // extract the e-mail address and display name from the nested anchor
          var anchor = fromValues[index].childNodes[0]; // fragile!!
          var displayName = gatherTextUnder(anchor);
          var image = document.createElementNS("http://www.w3.org/1999/xhtml", "img");
          image.className = 'conversation-contact-photo';
          // look up a card for the e-mail address
          var emailaddress = anchor.getAttribute('href').substr(mailtolength);
          var contacturl = generateContactPhotoUrl(displayName, '', abCardForEmailAddress(emailaddress,  Components.interfaces.nsIAbDirectory.opRead, {}), '', '') || 'chrome://messenger/skin/icons/compose-unknown-contact.png';
          image.setAttribute('src', contacturl);
          image.onerror = function ()
          {
            Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage("failed to load image: " + this.getAttribute('src'));
            this.setAttribute('src', 'chrome://messenger/skin/icons/compose-unknown-contact.png');
          }
          conversationPhotoCells[index].appendChild(image);
        }
        
        // only bail out if this is a conversation
        // and use fromValues.length (not this._conversation) to determine that.
        // see bug #4831 for details
        if (fromValues.length)
          return;
      }

      var msgHdr = null;
      // if we don't have any attachments, turn off the attachments flag
      var messageUrl = aUrl.QueryInterface(Components.interfaces.nsIMsgMessageUrl);
      try
      {
        msgHdr = messenger.msgHdrFromURI(messageUrl.uri);
      }
      catch (ex) {}
            
      try {
        if (!currentAttachments.length && msgHdr)
          msgHdr.markHasAttachments(false);
      }
      catch (ex) {}
      
      // clean up white space in block quotes
      cleanupWhiteSpaceInQuotes(aDocument);

      // mscott to do - hook this up to a separate observer
      gDataPane.onMessageParsed(currentAttachments, aUrl, aDocument);
      var attachmentIndicator = document.getElementById('pb-attachment-count-indicator');
      if (this._conversationMode)
        attachmentIndicator.attachmentCount = 0; // reset to avoid double counting since we are appending to currentAttachments
      setAttachmentHeader(currentAttachments, document.getElementById('pb-attachment-count-indicator'));
      updateAttachmentProperties(currentAttachments, aUrl);

      if (this._conversationMode)
      {        
        var quoteStyle = _determineQuoteStyle(aDocument);
        if (quoteStyle != kNoQuotes)
        {
          // get the conversation show quoted text element from the containing document
          // total hack abusing the name of the frameid
          var conversationIndex = aDocument.defaultView.name.slice(10 /* flyingpigs */);
          
          var autoRevealInlineQuotes = pref.getBoolPref("pb.auto_reveal_inline_quotes");
          if (quoteStyle == kInlineQuotes && autoRevealInlineQuotes)
          {
            // reveal inline quotes automatically
            // see bug #3413 for details
            _revealQuotesForDocument(aDocument, conversationIndex, true /* aReveal */, false /* aRecalcHeight */, false /* aShowInlineUI */);
          }
          else /* quoteStyle == kNonInlineQuotes) */
          {
            _revealQuotesForDocument(aDocument, conversationIndex, false /* aReveal */, false /* aRecalcHeight */, true /* aShowInlineUI */);
          }
        }
        
        // hack - now that the iframe for the message body has loaded, adjust the height of the document
        // to match the scroll height so we don't get any per message body scroll frames, the message body scrolls with the headers.
        msgPane.contentDocument.getElementById(aDocument.defaultView.name).style.height = 
          aDocument.documentElement.scrollHeight + "px";
      }

      OnMsgParsed(aUrl);
      _indexURLs(aUrl, aDocument);
    },

    onEndMsgHeaders: function(url)
    {
      OnMsgLoaded(url);
    },

    onMsgHasRemoteContent: function(aMsgHdr)
    {
      gMessageNotificationBar.setRemoteContentMsg(aMsgHdr);
    },

    mSecurityInfo  : null,
    get securityInfo()
    {
      return this.mSecurityInfo;
    },
    set securityInfo(aSecurityInfo)
    {
      this.mSecurityInfo = aSecurityInfo;
    },

    mDummyMsgHeader: null,

    get dummyMsgHeader()
    {
      if (!this.mDummyMsgHeader)
        this.mDummyMsgHeader = new nsDummyMsgHeader();
      return this.mDummyMsgHeader;
    },
    mProperties: null,
    get properties()
    {
      if (!this.mProperties)
        this.mProperties = Components.classes["@mozilla.org/hash-property-bag;1"].
          createInstance(Components.interfaces.nsIWritablePropertyBag2);
      return this.mProperties;
    }
};

function CreateSubjectFromMsgHdr(aMsgHdr)
{
  var decodedSubject = aMsgHdr.mime2DecodedSubject;
  return ((aMsgHdr.flags  & 0x0010 /* MSG_FLAG_HAS_RE */) ? "Re: " : "") + 
         (decodedSubject ? decodedSubject : gMessengerBundle.getString('noSubject'));
}

function SetTagHeader(aConversation)
{
  // it would be nice if we passed in the msgHdr from the back end
  var msgHdr;
  try
  {
    msgHdr = gDBView.hdrForFirstSelectedMessage;
  }
  catch (ex)
  {
    return; // no msgHdr to add our tags to
  }
  
  var tagKeys = aConversation ? tagKeysForThreadContainingMsgHdr(msgHdr, msgWindow) : tagKeysForMsgHdr(msgHdr);

  currentHeaderData.tags = {headerName: "tags", headerValue: tagKeys};
}

function CheckNotify()
{
  if ("NotifyClearAddresses" in this)
    NotifyClearAddresses();
}

function OnTagsChange()
{
  // rebuild the tag headers
  SetTagHeader(messageHeaderSink._conversationMode);

  // now update the expanded header view to rebuild the tags,
  // and then show or hide the tag header box.
  if (gBuiltHeaderView)
  {
    var headerEntry = gHeaderView.tags;
    if (headerEntry)
    {
      headerEntry.valid = ("tags" in currentHeaderData);
      if (headerEntry.valid)
        headerEntry.outputFunction(headerEntry, currentHeaderData.tags.headerValue);

      // if we are showing the expanded header view then we may need to collapse or
      // show the tag header box...
      headerEntry.enclosingBox.collapsed = !headerEntry.valid;
    }
  }
  // we really should have a separate onpropertyflagchange notification firing when there's a change to the keyword property on a msg hdr...
  // but for now this OnTagsChange method is the next best thing....update the tag unread counts
  if ("gTagUpdater" in window)
    gTagUpdater.updateFolder(GetFolderResource(GetFolderTree(), GetSelectedFolderIndex()));
}

// flush out any local state being held by a header entry for a given
// table
function ClearHeaderView(headerTable)
{
  for (index in headerTable)
  {
     var headerEntry = headerTable[index];
     if (headerEntry.useToggle)
       headerEntry.enclosingBox.childNodes[1].clearHeaderValues();

     headerEntry.valid = false;
  }
}

// make sure that any valid header entry in the table specified is
// visible
function showHeaderView(headerTable)
{
  // this is a total hack to work around a grid resizing issue. As we uncollapse rows in the grid, the header name column in the
  // grid doesn't always resize itself to adjust for the widest header name in the grid. Getting the boxObject width on the grid forces
  // the column width to get recalculated. Oddly enough this only seems to work if called before we uncollapse the rows in our for loop!
  // See pig Bug 1151 for details.
  var headerGridElement = document.getElementById('headerNameColumn');
  headerGridElement.boxObject.width;
  headerGridElement.boxObject.height;
  for (index in headerTable)
    headerTable[index].enclosingBox.collapsed = !headerTable[index].valid;
}

// enumerate through the list of headers and find the number that are visible
// add empty entries if we don't have the minimum number of rows
function EnsureMinimumNumberOfHeaders (headerTable)
{
  if (!gMinNumberOfHeaders) // 0 means we don't have a minimum..do nothing special
    return;

  var numVisibleHeaders = 0;
  for (index in headerTable)
  {
    if (headerTable[index].valid)
      numVisibleHeaders ++;
  }

  if (numVisibleHeaders < gMinNumberOfHeaders)
  {
    // how many empty headers do we need to add?
    var numEmptyHeaders = gMinNumberOfHeaders - numVisibleHeaders;

    // we may have already dynamically created our empty rows and we just need to make them visible
    for (index in headerTable)
    {
      if (index.indexOf("Dummy-Header") == 0 && numEmptyHeaders)
      {
        headerTable[index].valid = true;
        numEmptyHeaders--;
      }
    }

    // ok, now if we have any extra dummy headers we need to add, create a new header widget for them
    while (numEmptyHeaders)
    {
      var dummyHeaderId = "Dummy-Header" + gDummyHeaderIdIndex;
      gHeaderView[dummyHeaderId] = new createNewHeaderView(dummyHeaderId, "");
      gHeaderView[dummyHeaderId].valid = true;

      gDummyHeaderIdIndex++;
      numEmptyHeaders--;
    }

  }
}

// make sure the appropriate fields within the currently displayed view header mode
// are collapsed or visible...
function updateHeaderViews()
{
  if (gMinNumberOfHeaders)
    EnsureMinimumNumberOfHeaders(gHeaderView);
  showHeaderView(gHeaderView);
}

function OutputTags(headerEntry, headerValue)
{
  headerEntry.enclosingBox.childNodes[1].childNodes[1].headerValue = headerValue;
}

function OutputSubject(headerEntry, headerValue)
{
  headerEntry.enclosingBox.childNodes[1].childNodes[0].headerValue = headerValue;
  // now set a tooltip on the subject
  headerEntry.enclosingBox.childNodes[1].childNodes[0].setAttribute("tooltiptext", headerValue);
}

// default method for updating a header value into a header entry
function updateHeaderValue(headerEntry, headerValue)
{
  headerEntry.enclosingBox.childNodes[1].headerValue = headerValue;
}

function updateHeaderValueInTextNode(headerEntry, headerValue)
{
  headerEntry.textNode.value = headerValue;
}

function createNewHeaderView(headerName, label)
{
  // create a row
  var newRow = document.createElement('row');
  newRow.id = headerName + 'Box';
  newRow.collapsed = true;

  // setup the header name label
  var newHeaderName = document.createElement('label');
  newHeaderName.setAttribute('value', label);
  newHeaderName.className = 'headerName plain';
  newRow.appendChild(newHeaderName);

  // set up the header value
  var newHeaderValue = document.createElement('mailheadervalue');
  newRow.appendChild(newHeaderValue);

  // append the row to the grid containing our rows
  document.getElementById('headerRows').appendChild(newRow);

  this.enclosingBox = newRow;
  this.isValid = false;
  this.useToggle = false;
  this.outputFunction = updateHeaderValue;
}

// UpdateMessageHeaders: Iterate through all the current header data we received from mime for this message
// for each header entry table, see if we have a corresponding entry for that header. i.e. does the particular
// view care about this header value. if it does then call updateHeaderEntry
function UpdateMessageHeaders()
{
  // iterate over each header we received and see if we have a matching entry in each
  // header view table...

  var headerName;

  for (headerName in currentHeaderData)
  {
    var headerField = currentHeaderData[headerName];
    var headerEntry = null;

    if (headerName == "subject")
    {
      try {
        if (gDBView.keyForFirstSelectedMessage == nsMsgKey_None)
        {
          var folder = null;
          if (gCurrentFolderUri)
            folder = RDF.GetResource(gCurrentFolderUri).QueryInterface(Components.interfaces.nsIMsgFolder);
          setTitleFromFolder(folder, headerField.headerValue);
        }
      } catch (ex) {}
    }

    if (!gBuiltHeaderView)
    {
      if (headerName in gHeaderView)
       headerEntry = gHeaderView[headerName];
    } // if we are in expanded view....

    if (headerEntry)
    {
      if (headerName == "references" &&
          !(gShowReferences ||
            (gDBView.msgFolder && gDBView.msgFolder.server.type == "nntp")))
      {
        // hide references header if view all headers mode isn't selected, the pref show references is
        // deactivated and the currently displayed message isn't a newsgroup posting
        headerEntry.valid = false;
      }
      else
      {
        headerEntry.outputFunction(headerEntry, headerField.headerValue);
        headerEntry.valid = true;
      }
    }
  }

   gBuiltHeaderView = true;

  // now update the view to make sure the right elements are visible
  updateHeaderViews();

  if ("FinishEmailProcessing" in this)
    FinishEmailProcessing();
}

function ClearCurrentHeaders()
{
  currentHeaderData = {};
  currentAttachments = new Array();
}

function ShowMessageHeaderPane()
{
  gDataPane.show();

  document.getElementById('msgHeaderView').collapsed = false;

  /* workaround for 39655 */
  if (gFolderJustSwitched)
  {
    var el = document.getElementById("msgHeaderView");
    el.setAttribute("style", el.getAttribute("style"));
    gFolderJustSwitched = false;
  }
}

function HideMessageHeaderPane()
{
  document.getElementById('msgHeaderView').collapsed = true;

  // disable the File/Attachments menuitem
  document.getElementById("fileAttachmentMenu").setAttribute("disabled", "true");

  // notify the data pane
  gDataPane.hide();

  // if we are hiding the message pane, we need to disable the toggler
  // see bug #5930 for details
  goUpdateCommand("cmd_toggleInspectorPane");
}

function OutputNewsgroups(headerEntry, headerValue)
{
  headerValue = headerValue.replace(/,/g,", ");
  updateHeaderValue(headerEntry, headerValue);
}

// take string of message-ids separated by whitespace, split it
// into message-ids and send them together with the index number
// to the corresponding mail-messageids-headerfield element
function OutputMessageIds(headerEntry, headerValue)
{
  var messageIdArray = headerValue.split(/\s+/);

  headerEntry.enclosingBox.childNodes[1].clearHeaderValues();
  for (var i = 0; i < messageIdArray.length; i++)
    headerEntry.enclosingBox.childNodes[1].addMessageIdView(messageIdArray[i]);

  headerEntry.enclosingBox.childNodes[1].fillMessageIdNodes();
}

// OutputEmailAddresses --> knows how to take a comma separated list of email addresses,
// extracts them one by one, linkifying each email address into a mailto url.
// Then we add the link-ified email address to the parentDiv passed in.
//
// emailAddresses --> comma separated list of the addresses for this header field

function OutputEmailAddresses(headerEntry, emailAddresses)
{
  if (!emailAddresses)
    return;

  var emailAddressNode = headerEntry.enclosingBox.childNodes[1];
  // total hack to handle our special layout for From
  if (headerEntry.enclosingBox.id == 'fromBox')
    emailAddressNode = emailAddressNode.childNodes[0];

  if (msgHeaderParser)
  {
    var addresses = {};
    var fullNames = {};
    var names = {};
    var numAddresses =  0;

    numAddresses = msgHeaderParser.parseHeadersWithArray(emailAddresses, addresses, names, fullNames);
    var index = 0;
    while (index < numAddresses)
    {
      // if we want to include short/long toggle views and we have a long view, always add it.
      // if we aren't including a short/long view OR if we are and we haven't parsed enough
      // addresses to reach the cutoff valve yet then add it to the default (short) div.
      var address = {};
      address.emailAddress = addresses.value[index];
      address.fullAddress = fullNames.value[index];
      address.displayName = names.value[index];
      if (headerEntry.useToggle)
        emailAddressNode.addAddressView(address);
      else
        emailAddressNode.initialize(address.displayName, address.emailAddress, address.fullAddress);
      index++;
    }

    if (headerEntry.useToggle)
      emailAddressNode.buildViews();
  } // if msgheader parser
}

// createnewAttachmentInfo --> constructor method for creating new attachment object which goes into the
// data attachment array.
function createNewAttachmentInfo(contentType, url, displayName, uri, isExternalAttachment)
{
  this.contentType = contentType;
  this.url = url;
  this.displayName = displayName;
  this.uri = uri;
  this.isExternalAttachment = isExternalAttachment;
}

function saveAttachment(aAttachment)
{
  messenger.saveAttachment(aAttachment.contentType,
                           aAttachment.url,
                           encodeURIComponent(aAttachment.displayName),
                           aAttachment.uri, aAttachment.isExternalAttachment);
}

function openAttachment(aAttachment)
{
  messenger.openAttachment(aAttachment.contentType,
                           aAttachment.url,
                           encodeURIComponent(aAttachment.displayName),
                           aAttachment.uri, aAttachment.isExternalAttachment);
}

function detachAttachment(aAttachment, aSaveFirst)
{
  messenger.detachAttachment(aAttachment.contentType,
                           aAttachment.url,
                           encodeURIComponent(aAttachment.displayName),
                           aAttachment.uri, aSaveFirst);
}

function CanDetachAttachments()
{
  var uri = GetLoadedMessage();
  var canDetach = !IsNewsMessage(uri) && (!IsImapMessage(uri) || MailOfflineMgr.isOnline());
  if (canDetach && ("content-type" in currentHeaderData))
    canDetach = !ContentTypeIsSMIME(currentHeaderData["content-type"].headerValue);
  return canDetach;
}

/** Return true if the content type is an S/MIME one. */
function ContentTypeIsSMIME(contentType)
{
  // S/MIME is application/pkcs7-mime and application/pkcs7-signature
  // - also match application/x-pkcs7-mime and application/x-pkcs7-signature.
  return /application\/(x-)?pkcs7-(mime|signature)/.test(contentType);
}

function MessageIdClick(node, event)
{
  if (event.button == 0)
  {
    var messageId = GetMessageIdFromNode(node, true);
    OpenMessageForMessageId(messageId);
  }
}

function cloneAttachment(aAttachment)
{
  var obj = new Object();
  obj.contentType = aAttachment.contentType;
  obj.url = aAttachment.url;
  obj.displayName = aAttachment.displayName;
  obj.uri = aAttachment.uri;
  obj.isExternalAttachment = aAttachment.isExternalAttachment;
  return obj;
}

function createAttachmentDisplayName(aAttachment)
{
  // Strip any white space at the end of the display name to avoid
  // attachment name spoofing (especially Windows will drop trailing dots
  // and whitespace from filename extensions). Leading and internal
  // whitespace will be taken care of by the crop="center" attribute.
  // We must not change the actual filename, though.
  return aAttachment.displayName.replace(/\s+$/, "");
}

// attachment --> the attachment struct containing all the information on the attachment
// listitem --> the listitem currently showing the attachment.
function setApplicationIconForAttachment(attachment, listitem, largeView)
{
  var iconSize = largeView ? kLargeIcon : kSmallIcon;
  listitem.setAttribute('image', GetIconURLForFile(attachment.displayName, attachment.contentType, iconSize));
}    

// Public method called when we create the attachments file menu
function FillAttachmentListPopup(popup)
{
  // the FE sometimes call this routine TWICE...I haven't been able to figure out why yet...
  // protect against it...
  if (!gBuildAttachmentPopupForCurrentMsg) return;

  var attachmentIndex = 0;

  // otherwise we need to build the attachment view...
  // First clear out the old view...
  ClearAttachmentMenu(popup);

  for (index in currentAttachments)
  {
    ++attachmentIndex;
    addAttachmentToPopup(popup, currentAttachments[index], attachmentIndex);
  }

  gBuildAttachmentPopupForCurrentMsg = false;
  var detachAllMenu = document.getElementById('file-detachAllAttachments');
  var deleteAllMenu = document.getElementById('file-deleteAllAttachments');
  if (CanDetachAttachments())
  {
    detachAllMenu.removeAttribute('disabled');
    deleteAllMenu.removeAttribute('disabled');
  }
  else
  {
    detachAllMenu.setAttribute('disabled', 'true');
    deleteAllMenu.setAttribute('disabled', 'true');
  }
}

// Public method used to clear the file attachment menu
function ClearAttachmentMenu(popup)
{
  if ( popup )
  {
     while ( popup.childNodes[0].localName == 'menu' )
       popup.removeChild(popup.childNodes[0]);
  }
}

// Public method used to determine the number of attachments for the currently displayed message...
function GetNumberOfAttachmentsForDisplayedMessage()
{
  return currentAttachments.length;
}

// private method used to build up a menu list of attachments
function addAttachmentToPopup(popup, attachment, attachmentIndex)
{
  if (popup)
  {
    var item = document.createElement('menu');
    if ( item )
    {
      if (!gMessengerBundle)
        gMessengerBundle = document.getElementById("bundle_messenger");

      // insert the item just before the separator...the separator is the 2nd to last element in the popup.
      item.setAttribute('class', 'menu-iconic');
      setApplicationIconForAttachment(attachment,item, false);

      var numItemsInPopup = popup.childNodes.length;
      // find the separator
      var indexOfSeparator = 0;
      while (popup.childNodes[indexOfSeparator].localName != 'menuseparator')
        indexOfSeparator++;

      var displayName = createAttachmentDisplayName(attachment);
      var formattedDisplayNameString = gMessengerBundle.getFormattedString("attachmentDisplayNameFormat",
                                       [attachmentIndex, displayName]);

      item.setAttribute("crop", "center");
      item.setAttribute('label', formattedDisplayNameString);
      item.setAttribute('accesskey', attachmentIndex);

      // each attachment in the list gets its own menupopup with options for saving, deleting, detaching, etc.
      var openpopup = document.createElement('menupopup');
      openpopup = item.appendChild(openpopup);

      // Due to Bug #314228, we must append our menupopup to the new attachment menu item
      // before we inserting the attachment menu into the popup. If we don't, our attachment
      // menu items will not show up.
      item = popup.insertBefore(item, popup.childNodes[indexOfSeparator]);

      var menuitementry = document.createElement('menuitem');
      menuitementry.attachment = cloneAttachment(attachment);
      menuitementry.setAttribute('oncommand', 'openAttachment(this.attachment)');

      if (!gSaveLabel)
        gSaveLabel = gMessengerBundle.getString("saveLabel");
      if (!gSaveLabelAccesskey)
        gSaveLabelAccesskey = gMessengerBundle.getString("saveLabelAccesskey");
      if (!gOpenLabel)
        gOpenLabel = gMessengerBundle.getString("openLabel");
      if (!gOpenLabelAccesskey)
        gOpenLabelAccesskey = gMessengerBundle.getString("openLabelAccesskey");
      if (!gDetachLabel)
        gDetachLabel = gMessengerBundle.getString("detachLabel");
      if (!gDetachLabelAccesskey)
        gDetachLabelAccesskey = gMessengerBundle.getString("detachLabelAccesskey");
      if (!gDeleteLabel)
        gDeleteLabel = gMessengerBundle.getString("deleteLabel");
      if (!gDeleteLabelAccesskey)
        gDeleteLabelAccesskey = gMessengerBundle.getString("deleteLabelAccesskey");

      // we should also check if an attachment has been detached...
      // but that uses X-Mozilla-External-Attachment-URL, which
      // we'd need to check for somehow.

      var signedOrEncrypted = false;
      if ("content-type" in currentHeaderData)
      {
        var contentType = currentHeaderData["content-type"].headerValue;
        signedOrEncrypted = contentType.indexOf("application/x-pkcs7-mime") >= 0 ||
            contentType.indexOf("application/x-pkcs7-signature") >= 0;
      }
      var canDetach = !(/news-message:/.test(attachment.uri)) &&
          !signedOrEncrypted &&
          (!(/imap-message/.test(attachment.uri)) || MailOfflineMgr.isOnline());
      menuitementry.setAttribute('label', gOpenLabel);
      menuitementry.setAttribute('accesskey', gOpenLabelAccesskey);
      menuitementry = openpopup.appendChild(menuitementry);
      if (attachment.contentType == 'text/x-moz-deleted')
        menuitementry.setAttribute('disabled', true);

      var quickLookService = Components.classes["@postbox-inc.com/mail/quicklook;1"].getService(Components.interfaces.pbIQuickLookService);
      if (quickLookService.isAvailable)
      {
        menuitementry = document.createElement('menuitem');
        menuitementry.attachment = cloneAttachment(attachment);
        menuitementry.setAttribute('oncommand', "gDataPane.handleAttachment(this, 'quicklook', null);");
#ifdef XP_MACOSX
        menuitementry.setAttribute('label', gMessengerBundle.getString("pbMsgHdrViewOverlay.attachment.quicklookMac.label"));
        menuitementry.setAttribute('accesskey', gMessengerBundle.getString("pbMsgHdrViewOverlay.attachment.quicklookMac.accesskey"));
#else
        menuitementry.setAttribute('label', gMessengerBundle.getString("pbMsgHdrViewOverlay.attachment.quicklook.label"));
        menuitementry.setAttribute('accesskey', gMessengerBundle.getString("pbMsgHdrViewOverlay.attachment.quicklook.accesskey"));
#endif
        menuitementry = openpopup.appendChild(menuitementry);
        if (attachment.contentType == 'text/x-moz-deleted')
          menuitementry.setAttribute('disabled', true);
      }

      var menuseparator = document.createElement('menuseparator');
      openpopup.appendChild(menuseparator);

      menuitementry = document.createElement('menuitem');
      menuitementry.attachment = cloneAttachment(attachment);
      menuitementry.setAttribute('oncommand', 'saveAttachment(this.attachment)');
      menuitementry.setAttribute('label', gSaveLabel);
      menuitementry.setAttribute('accesskey', gSaveLabelAccesskey);
      if (attachment.contentType == 'text/x-moz-deleted')
        menuitementry.setAttribute('disabled', true);
      menuitementry = openpopup.appendChild(menuitementry);

      var menuseparator = document.createElement('menuseparator');
      openpopup.appendChild(menuseparator);

      menuitementry = document.createElement('menuitem');
      menuitementry.attachment = cloneAttachment(attachment);
      menuitementry.setAttribute('oncommand', 'detachAttachment(this.attachment, true)');
      menuitementry.setAttribute('label', gDetachLabel);
      menuitementry.setAttribute('accesskey', gDetachLabelAccesskey);
      if (attachment.contentType == 'text/x-moz-deleted' || !canDetach)
        menuitementry.setAttribute('disabled', true);
      menuitementry = openpopup.appendChild(menuitementry);

      menuitementry = document.createElement('menuitem');
      menuitementry.attachment = cloneAttachment(attachment);
      menuitementry.setAttribute('oncommand', 'detachAttachment(this.attachment, false)');
      menuitementry.setAttribute('label', gDeleteLabel);
      menuitementry.setAttribute('accesskey', gDeleteLabelAccesskey);
      if (attachment.contentType == 'text/x-moz-deleted' || !canDetach)
        menuitementry.setAttribute('disabled', true);
      menuitementry = openpopup.appendChild(menuitementry);
    }  // if we created a menu item for this attachment...
  } // if we have a popup
}

function HandleAllAttachments(action)
{
 try
 {
   // convert our attachment data into some c++ friendly structs
   var attachmentContentTypeArray = new Array();
   var attachmentUrlArray = new Array();
   var attachmentDisplayNameArray = new Array();
   var attachmentMessageUriArray = new Array();

   // populate these arrays..
   var actionIndex = 0;
   for (var index in currentAttachments)
   {
     // exclude all attachments already deleted
     var attachment = currentAttachments[index];
     if ( attachment.contentType != 'text/x-moz-deleted' )
     {
       attachmentContentTypeArray[actionIndex] = attachment.contentType;
       attachmentUrlArray[actionIndex] = attachment.url;
       attachmentDisplayNameArray[actionIndex] = encodeURI(attachment.displayName);
       attachmentMessageUriArray[actionIndex] = attachment.uri;
       ++actionIndex;
     }
   }

   // okay the list has been built... now call our action code...
   if ( action == 'save' )
     messenger.saveAllAttachments(attachmentContentTypeArray.length,
                                  attachmentContentTypeArray, attachmentUrlArray,
                                  attachmentDisplayNameArray, attachmentMessageUriArray);
   else if ( action == 'detach' )
     messenger.detachAllAttachments(attachmentContentTypeArray.length,
                                    attachmentContentTypeArray, attachmentUrlArray,
                                    attachmentDisplayNameArray, attachmentMessageUriArray,
                                    true); // save
   else if ( action == 'delete' )
     messenger.detachAllAttachments(attachmentContentTypeArray.length,
                                    attachmentContentTypeArray, attachmentUrlArray,
                                    attachmentDisplayNameArray, attachmentMessageUriArray,
                                    false); // don't save
   else
     dump ("** unknown HandleAllAttachments action: " + action + "**\n");
   }
   catch (ex)
   {
   dump ("** failed to handle all attachments **\n");
   }
}

function ClearAttachmentList()
{
  // we also have to disable the File/Attachments menuitem
  var node = document.getElementById("fileAttachmentMenu");
  if (node)
    node.setAttribute("disabled", "true");
}

// CopyWebsiteAddress takes the website address title button, extracts
// the website address we stored in there and copies it to the clipboard
function CopyWebsiteAddress(websiteAddressNode)
{
  if (websiteAddressNode)
  {
    var websiteAddress = websiteAddressNode.getAttribute("value");

    var contractid = "@mozilla.org/widget/clipboardhelper;1";
    var iid = Components.interfaces.nsIClipboardHelper;
    var clipboard = Components.classes[contractid].getService(iid);
    clipboard.copyString(websiteAddress);
  }
}

function nsDummyMsgHeader()
{
}

nsDummyMsgHeader.prototype =
{
  mProperties : new Array,
  getStringProperty : function(property) {return (this.mProperties[property] ? this.mProperties[property] : "");},
  setStringProperty : function(property, val) {this.mProperties[property] = val;},
  messageSize : 0,
  recipients : null,
  author : null,
  subject : null,
  ccList : null,
  messageId : null,
  accountKey : "",
  folder : null,
  get mime2DecodedSubject() { return this.subject; },
  get mime2DecodedRecipients() { return this.recipients; } ,
  get mime2DecodedAuthor() { return this.author; },
  get mime2DecodedCcList() { return this.ccList; },
  markHasAttachments : function(hasAttachments) { /* see bug #1221 for details */ }
};


function isImage(aFileName, aContentType) 
{
  var filesuffix = aFileName.substr(aFileName.lastIndexOf(".")).toLowerCase();
  // XXX todo stop duplicating this image/* logic
  return (aContentType == 'image/jpeg' || aContentType == 'image/jpg' || aContentType == 'image/png' || aContentType == 'image/pjpeg' || aContentType == 'image/gif' ||
          filesuffix == '.jpeg' || filesuffix == '.jpg' || filesuffix == '.png' || filesuffix == '.pjpeg' || filesuffix == '.gif');
}

#include pbMsgHdrViewOverlay.js

